from datetime import timedelta
from django.utils import timezone
from django.apps import apps
from .exceptions import (
    ConfigIsNotDict,
    InvalidJwtAlgorithm,
    InvalidLocation,
    InvalidExpires,
)

allowed_algorithm = ('HS256',)
allowed_location = ('headers',)

class ConfigParser:

    def __init__(self, config: dict):
        if not isinstance(config, dict):
            raise ConfigIsNotDict()
        self.jwt_algorithm = self.validate_jwt_algorithm(config)
        self.token_location = self.validate_token_location(config)
        self.access_token_expires = self.validate_access_token_expires(config)
        self.refresh_token_expires = self.validate_refresh_token_expires(config)

    @staticmethod
    def validate_jwt_algorithm(config: dict):
        jwt_algorithm = config.get('ALGORITHM', 'HS256')
        if jwt_algorithm not in allowed_algorithm:
            raise InvalidJwtAlgorithm(jwt_algorithm, allowed_algorithm)
        return jwt_algorithm

    @staticmethod
    def validate_token_location(config: dict):
        token_location = config.get('LOCATION', ['headers'])
        if not (
            isinstance(token_location, list)
            and not (set(token_location) - set(allowed_location))
        ):
            raise InvalidLocation(token_location, allowed_location)
        return token_location

    @staticmethod
    def validate_access_token_expires(config: dict):
        expires = config.get('ACCESS_TOKEN_EXPIRES', timedelta(days=2))
        if isinstance(expires, int) and expires > 0:
            expires = timedelta(seconds=expires)
        if not isinstance(expires, timedelta):
            raise InvalidExpires('ACCESS_TOKEN')
        return expires

    @staticmethod
    def validate_refresh_token_expires(config: dict):
        expires = config.get('REFRESH_TOKEN_EXPIRES', timedelta(days=30))
        if isinstance(expires, int) and expires > 0:
            expires = timedelta(seconds=expires)
        if not isinstance(expires, timedelta):
            raise InvalidExpires('REFRESH_TOKEN')
        return expires





from datetime import timedelta
from django.apps import AppConfig
from django.conf import settings
from .config import ConfigParser
from .exceptions import NotFoundSecretKey


class DjangoJwtExtConfig(AppConfig):
    name = 'django_jwt_extended'
    verbose_name = "Django JWT Extended"

    def ready(self):
        if not hasattr(settings, 'SECRET_KEY'):
            raise NotFoundSecretKey()

        data = ConfigParser(
            settings.JWT_CONFIG
            if hasattr(settings, 'JWT_CONFIG')
            else {}
        )
        self.jwt_algorithm = data.jwt_algorithm
        self.token_location = data.token_location
        self.access_token_expires = data.access_token_expires
        self.refresh_token_expires = data.refresh_token_expires
        self.token_header_name = 'Authorization'

        # default error messages
        self.jwt_not_found_msg = {
            'msg': 'JWT token not found'
        }
        self.bearer_error_msg = {
            'msg': (
                f"Missing 'Bearer' type in "
                f"'{self.token_header_name}' header."
				f" Expected '{self.token_header_name}: "
                f"Bearer <JWT>'"
            )
        }
        self.decode_error_msg = {
            'msg': 'Signature verification failed.'
        }
        self.expired_token_msg = {
            'msg': 'JWT Token has expired'
        }
        self.invalid_token_type_msg = {
            'msg': "Invalid JWT token type"
        }
        self.invalid_nbf_msg = {
            'msg': "The token is not yet valid (nbf)"
        }


from .exceptions import NotFoundRequest
from django.http import HttpRequest
from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
try:
    # If use django_rest_framework,
    from rest_framework.request import Request as RestRequest
    REQUESTS = (HttpRequest, ASGIRequest, WSGIRequest, RestRequest)
except:
    REQUESTS = (HttpRequest, ASGIRequest, WSGIRequest)


def _find_request_object(*args, **kwargs):
    for object in args:
        if any([isinstance(object, req) for req in REQUESTS]):
            return object
    for object in kwargs.values():
        if any([isinstance(object, req) for req in REQUESTS]):
            return object
    return None